// naumann@stce.rwth-aachen.de

#include<fstream>

template<typename T>
list<T>::list::list_element::list_element(const T& d) 
  : data(d), next(nullptr) {}

template<typename T>
list<T>::list(int length) : l(length) {
  if (length<=0) throw(std::runtime_error("list: length<=0"));
  list_element* last=new list_element(T());
  first=last;
  for (int i=1;i<length;i++) {
    last->next=new list_element(T());
    last=last->next;
  }
}

template<typename T>
void list<T>::free(const list_element *e) {
  if (e->next) free(e->next);
  delete e; 
}

template<typename T>
list<T>::~list() { free(first); }

/*
template<typename T>
list<T>::~list() {
  while (first) {
    list_element* current=first;
    first=first->next;
    delete current;
  }
}

template<typename T>
list<T>::~list() {
  for (int i=0;i<l;i++) {
    list_element* current=first;
    first=first->next;
    delete current;
  }
}
*/

template<typename T>
int list<T>::length() const { return l; }

template<typename T>
void list<T>::erase(int pos) {
  if (pos<0||pos>=l) throw(std::runtime_error("erase: pos<0||pos>=l"));
  list_element* current=first;
  if (pos==0) {
    first=first->next;
    delete current; 
  } else {
    for (int i=1;i<=pos-1;i++) current=current->next;
    list_element* old_element=current->next;
    current->next=old_element->next;
    delete old_element;
  }
   l--;
}

template<typename T>
void list<T>::insert(int pos, const T& dat) {
  if (pos<0||pos>=l) throw(std::runtime_error("insert: pos<0||pos>=l"));
  list_element* new_element=new list_element(dat);
  if (pos==0) {
    new_element->next=first;
    first=new_element;
  } else {
    list_element* current=first;
    for (int i=1;i<=pos-1;i++) current=current->next;
    new_element->next=current->next; 
    current->next=new_element;
  }
  l++;
}

template<typename T>
void list<T>::set(int pos, const T& dat) {
  if (pos<0||pos>=l) throw(std::runtime_error("erase: pos<0||pos>=l"));
  list_element* current=first;
  for (int i=1;i<=pos;i++) current=current->next;
  current->data=dat;
}

template<typename T>
const T& list<T>::get(int pos) const {
  if (pos<0||pos>=l) throw(std::runtime_error("erase: pos<0||pos>=l"));
  list_element* current=first;
  for (int i=1;i<=pos;i++) current=current->next;
  return current->data;
}

template<typename T>
void list<T>::to_dot(const char* filename) const {
  std::ofstream out(filename);
  out << "digraph {" << std::endl;
  out << "rankdir=\"LR\";" << std::endl;
  list_element* current=first;
  for (int i=0;i<l;i++,current=current->next) 
    out << i << " [label=\"" << current->data << "\"];" << std::endl;
  for (int i=0;i<l-1;i++) 
    out << i << "->" << i+1 << ";" << std::endl;
  out << "}" << std::endl;
  out.close();
}
