// naumann@stce.rwth-aachen.de

#include<fstream>
#include<algorithm>

template<typename T>
list<T>::list(int len) {
  for (int i=0;i<len;i++) data.push_back(T());
}

template<typename T>
int list<T>::length() const { return data.size(); }

template<typename T>
void list<T>::erase(int pos) {
  if (pos<0||pos>=length()) 
    throw(std::runtime_error("erase: pos<0||pos>=length"));
  auto data_it=data.begin(); 
  for (int i=1;i<=pos;i++,data_it++);
  data.erase(data_it);
}

template<typename T>
void list<T>::insert(int pos, const T& dat) {
  if (pos<0||pos>=length()) 
    throw(std::runtime_error("insert: pos<0||pos>=length"));
  auto data_it=data.begin(); 
  for (int i=1;i<=pos;i++,data_it++);
  data.insert(data_it,dat);
}

template<typename T>
void list<T>::set(int pos, const T& dat) {
  if (pos<0||pos>=length()) 
    throw(std::runtime_error("set: pos<0||pos>=length"));
  auto data_it=data.begin(); 
  for (int i=1;i<=pos;i++,data_it++);
  *data_it=dat;
}

template<typename T>
const T& list<T>::get(int pos) const {
  if (pos<0||pos>=length()) 
    throw(std::runtime_error("get: pos<0||pos>=length"));
  auto data_it=data.cbegin(); 
  for (int i=1;i<=pos;i++,data_it++);
  return *data_it;
}

template<typename T>
void list<T>::algorithms() {
  std::cout << "count(42): " << count(data.begin(),data.end(),42) << std::endl;
  std::cout << "min: " << *min_element(data.begin(),data.end()) << std::endl;
  std::cout << "max: " << *max_element(data.begin(),data.end()) << std::endl;
  std::cout << "before sorting: ";
  for (int i=0;i<length();i++) std::cout << get(i) << " ";
  std::cout << std::endl;
  data.sort();
  std::cout << "after sorting: ";
  for (int i=0;i<length();i++) std::cout << get(i) << " ";
  std::cout << std::endl;
  std::cout << "permutations: " << std::endl;
  int c=0;
  do {
    std::cout << c++ << ": ";
    for (auto i=data.cbegin();i!=data.cend();i++)
      std::cout << *i << " ";
    std::cout << std::endl;
  } while (next_permutation(data.begin(),data.end()));
}

template<typename T>
void list<T>::to_dot(const char* filename) const {
  std::ofstream out(filename);
  out << "digraph {" << std::endl;
  out << "rankdir=\"LR\";" << std::endl;
  size_t i=0;
  for (auto data_it=data.begin();data_it!=data.end();data_it++,i++)
    out << i << " [label=\"" << *data_it << "\"];" << std::endl;
  if (i>1)
    for (;i>1;i--) out << i-2 << "->" << i-1 << ";" << std::endl;
  out << "}" << std::endl;
  out.close();
}
